// GLSL verze 1.0 a Shader model 4.0
#version 120
#extension GL_EXT_gpu_shader4 : enable

// Textury pozic a normal
uniform sampler2D final_position;
uniform sampler2D final_normal;

// Vstup z geometry shader
varying in vec2 v2_texCoord_gout;
varying in vec3 lightPosMV_gout;

// Velikost 
const vec2 mapScale = vec2(1.0 / 1024.0, 1.0 / 768.0);

// Main
void main()
{
	// Normaly a pozice na danem pixelu
	vec3 normalTex = texture2D(final_normal, v2_texCoord_gout).xyz;
	vec3 positionTex = texture2D(final_position, v2_texCoord_gout).xyz;

	// Vypocet normalizovaneho vektoru ze svetla k bodu na kterem zrovna pocitame
	vec3 lightDir = normalize(lightPosMV_gout - positionTex);
	// Lambertuv zakon kosinu, viz kapitola implementace
	float diffuseLight = max(dot(lightDir, normalize(normalTex)), 0.0);
	// Pridame nejake rozptylene svetlo
	// x, y ci z se muze rovnat nule i na bode kde poloprimka mine teleso
	if(positionTex.x != 0.0 || positionTex.y != 0.0 || positionTex.z != 0.0)
	{
		diffuseLight = min(diffuseLight + 0.3, 1.0);
	}
	// Celkove svetlo ztlumime, aby vynikl obrys
	diffuseLight *= 0.5;

	// Suma vertikalni matice sobelova operatoru
	vec3 Gx = vec3(0.0, 0.0, 0.0);
	Gx += texture2D(final_normal, v2_texCoord_gout + vec2(-mapScale.x, mapScale.y)).xyz;
	Gx += 2.0 * texture2D(final_normal, v2_texCoord_gout + vec2(0.0, mapScale.y)).xyz;
	Gx += texture2D(final_normal, v2_texCoord_gout + vec2(mapScale.x, mapScale.y)).xyz;
	Gx -= texture2D(final_normal, v2_texCoord_gout + vec2(-mapScale.x, -mapScale.y)).xyz;
	Gx -= 2.0 * texture2D(final_normal, v2_texCoord_gout + vec2(0.0, -mapScale.y)).xyz;
	Gx -= texture2D(final_normal, v2_texCoord_gout + vec2(mapScale.x, -mapScale.y)).xyz;

	// Suma horizontalni matice sobelova operatoru
	vec3 Gy = vec3(0.0, 0.0, 0.0);
	Gy += texture2D(final_normal, v2_texCoord_gout + vec2(-mapScale.x, -mapScale.y)).xyz;
	Gy += 2.0 * texture2D(final_normal, v2_texCoord_gout + vec2(-mapScale.x, 0.0)).xyz;
	Gy += texture2D(final_normal, v2_texCoord_gout + vec2(-mapScale.x, mapScale.y)).xyz;
	Gy -= texture2D(final_normal, v2_texCoord_gout + vec2(mapScale.x, -mapScale.y)).xyz;
	Gy -= 2.0 * texture2D(final_normal, v2_texCoord_gout + vec2(mapScale.x, 0.0)).xyz;
	Gy -= texture2D(final_normal, v2_texCoord_gout + vec2(mapScale.x, mapScale.y)).xyz;

	// Prepis sobelova operatoru do intenzity hrany
	float sobel_operator = abs(Gx.x) + abs(Gx.y) + abs(Gx.z) + abs(Gy.x) + abs(Gy.y) + abs(Gy.z);
	// A odecteni male hodnoty, abychom zabranili chybam v obrysu
	sobel_operator = clamp(sobel_operator - 1.0, 0.0, 1.0);

	// Vysledna barva pixelu je soucet sobelova operatoru a dopadoveho svetla
	gl_FragColor = vec4(sobel_operator + diffuseLight, sobel_operator + diffuseLight, sobel_operator + diffuseLight, 1.0);
}